#include <avr/io.h>
#include <stdlib.h>
#include "lcd.h"
#include "keyb.h"

struct disp_data {
	uint8_t upperLine[16];
	uint8_t lowerLine[16];
};

void updateDisplay(struct disp_data * displayData);
void shiftLeft(struct disp_data * displayData, uint8_t playerPosition);
uint8_t generateObstacle(struct disp_data * displayData);
void processKeyboard(struct disp_data * displayData, uint8_t * playerPosition);

int main(void)
{
	lcdInit();
	keybInit();
	// Timer1 w trybie CTC, preskaler /1024
	TCCR1B = _BV(CS12) | _BV(CS10) | _BV(WGM12);
	OCR1A = 8000;
	lcdString("Press key");
	lcdGotoXY(6, 1);
	lcdString("to play...");
	getKey();
	// inicjalizacja generatora pseudolosowego
	srand(TCNT1);
	struct disp_data displayData;
	while(1) {
		// wyczyszczenie danych wywietlacza
		for (uint8_t i = 0; i < 16; i++) {
			displayData.upperLine[i] = ' ';
			displayData.lowerLine[i] = ' ';
		}
		uint8_t playerPosition = 0;
		uint8_t obstacleGenerated = 0;
		while (1) 
		{
			// generowanie przeszkody
			if (!obstacleGenerated) {
				obstacleGenerated = generateObstacle(&displayData);
			} else {
				obstacleGenerated = 0;
			}
			// opnienie i odczyt klawiatury
			while(!(TIFR & _BV(OCF1A))) {
				processKeyboard(&displayData, &playerPosition);
			}
			TIFR |= _BV(OCF1A);
			// wykrycie kolizji i zakoczenie gry
			if ((playerPosition == 0 && displayData.upperLine[1] == 'X') ||
			 (playerPosition == 1 && displayData.lowerLine[1] == 'X')) {
				lcdGotoXY(3, 0);
				lcdString("GAME OVER");
				getKey();
				break;
			}
			// przesunicie przeszkd w lewo
			shiftLeft(&displayData, playerPosition);
			// wysanie stanu gry na wywietlacz
			updateDisplay(&displayData);
		}
	}
}

void updateDisplay(struct disp_data * displayData) {
	lcdGotoXY(0, 0);
	for (uint8_t i = 0; i < 16; i++) lcdWriteData(displayData->upperLine[i]);
	lcdGotoXY(0, 1);
	for (uint8_t i = 0; i < 16; i++) lcdWriteData(displayData->lowerLine[i]);
}

void shiftLeft(struct disp_data * displayData, uint8_t playerPosition) {
	for (uint8_t i = 0; i < 15; i++) {
		displayData->upperLine[i] = displayData->upperLine[i + 1];
		displayData->lowerLine[i] = displayData->lowerLine[i + 1];
	}
	displayData->upperLine[15] = ' ';
	displayData->lowerLine[15] = ' ';
	if (playerPosition) displayData->lowerLine[0] = '>'; else displayData->upperLine[0] = '>';
}

uint8_t generateObstacle(struct disp_data * displayData) {
	int randomNumber = rand();
	if (randomNumber % 2) {
		randomNumber = rand();
		if (randomNumber % 2) {
			displayData->upperLine[15] = 'X';
			return 1;
		} else {
			displayData->upperLine[15] = ' ';
			return 0;
		}
	} else {
		randomNumber = rand();
		if (randomNumber % 2) {
			displayData->lowerLine[15] = 'X';
			return 1;
		} else {
			displayData->lowerLine[15] = ' ';
			return 0;
		}
	}
}

void processKeyboard(struct disp_data * displayData, uint8_t * playerPosition) {
	uint8_t key = readKeyboard();
	if (key) {
		if (key == 1) {
			*playerPosition = 0;
			displayData->upperLine[0] = '>';
			if (displayData->lowerLine[0] == '>') displayData->lowerLine[0] = ' ';
		}
		if (key == 5) {
			*playerPosition = 1;
			displayData->lowerLine[0] = '>';
			if (displayData->upperLine[0] == '>') displayData->upperLine[0] = ' ';
		}
		updateDisplay(displayData);
	}
}